"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllProductVariantIds = exports.getAllProductImages = exports.productExists = exports.createProduct = exports.getProductIdsFromVariantIds = exports.getProductIdFromVariantId = exports.deleteProduct = exports.deleteProductMutation = exports.bulkUpdateProducts = exports.updateProductMutation = exports.bulkAppendProductsImages = exports.appendProductImagesMutation = exports.bulkUpdateProductsImages = exports.updateProductImagesMutation = exports.bulkUpdateProductsTags = exports.updateProductTagsMutation = exports.bulkCreateProducts = exports.createProductMutation = void 0;
const api_1 = require("./api");
const bulk_operations_1 = require("./bulk-operations");
const promises_1 = require("timers/promises");
exports.createProductMutation = (0, api_1.gql) `
  mutation productCreate($input: ProductInput!) {
    productCreate(input: $input) {
      product {
        id
        handle
        description
        createdAt
        status
        totalVariants
        variants(first: 1) {
          edges {
            node {
              id
              sku
              inventoryItem {
                id
              }
            }
          }
        }
      }
      userErrors {
        field
        message
      }
    }
  }
`;
async function bulkCreateProducts(JSONLReadStream, fileName) {
    return (0, bulk_operations_1.bulkUpdate)({
        JSONLReadStream,
        mutation: exports.createProductMutation,
        fileName,
    });
}
exports.bulkCreateProducts = bulkCreateProducts;
exports.updateProductTagsMutation = (0, api_1.gql) `
  mutation productUpdate($input: ProductInput!) {
    productUpdate(input: $input) {
      product {
        id
        tags
      }
      userErrors {
        field
        message
      }
    }
  }
`;
async function bulkUpdateProductsTags(JSONLReadStream, fileName) {
    return (0, bulk_operations_1.bulkUpdate)({
        JSONLReadStream,
        mutation: exports.updateProductTagsMutation,
        fileName,
    });
}
exports.bulkUpdateProductsTags = bulkUpdateProductsTags;
exports.updateProductImagesMutation = (0, api_1.gql) `
  mutation productUpdate($input: ProductInput!) {
    productUpdate(input: $input) {
      product {
        id
        images {
          edges {
            node {
              id
              altText
              url
            }
          }
        }
      }
      userErrors {
        field
        message
      }
    }
  }
`;
async function bulkUpdateProductsImages(JSONLReadStream, fileName) {
    return (0, bulk_operations_1.bulkUpdate)({
        JSONLReadStream,
        mutation: exports.updateProductImagesMutation,
        fileName,
    });
}
exports.bulkUpdateProductsImages = bulkUpdateProductsImages;
exports.appendProductImagesMutation = (0, api_1.gql) `
  mutation productAppendImages($input: ProductAppendImagesInput!) {
    productAppendImages(input: $input) {
      newImages {
        id
        altText
        url
      }
      product {
        id
      }
      userErrors {
        field
        message
      }
    }
  }
`;
async function bulkAppendProductsImages(JSONLReadStream, fileName) {
    return (0, bulk_operations_1.bulkUpdate)({
        JSONLReadStream,
        mutation: exports.appendProductImagesMutation,
        fileName,
    });
}
exports.bulkAppendProductsImages = bulkAppendProductsImages;
exports.updateProductMutation = (0, api_1.gql) `
  mutation productUpdate($input: ProductInput!) {
    productUpdate(input: $input) {
      product {
        id
        tags
        id
        handle
        description
        createdAt
        status
        totalVariants
        variants(first: 1) {
          edges {
            node {
              id
              sku
              inventoryItem {
                id
              }
            }
          }
        }
      }
      }
      userErrors {
        field
        message
      }
    }
  }
`;
async function bulkUpdateProducts(JSONLReadStream, fileName) {
    return (0, bulk_operations_1.bulkUpdate)({
        JSONLReadStream,
        mutation: exports.updateProductMutation,
        fileName,
    });
}
exports.bulkUpdateProducts = bulkUpdateProducts;
exports.deleteProductMutation = `
mutation productDelete($input: ProductDeleteInput!) {
  productDelete(input: $input) {
    deletedProductId
    userErrors {
      field
      message
    }
  }
}
`;
async function deleteProduct(input) {
    const formattedInput = typeof input === "string" ? { input: { id: input } } : input;
    return await api_1.graphqlClient.request(exports.deleteProductMutation, formattedInput);
}
exports.deleteProduct = deleteProduct;
async function getProductIdFromVariantId(variantId) {
    try {
        const res = await api_1.graphqlClient.request(`
        {
          productVariant(id: "gid://shopify/ProductVariant/${variantId}") {
            product {
              id
            }
          }
        }
      `);
        const map = {};
        map[variantId] = res.productVariant.product.id;
        return map;
    }
    catch (e) {
        throw e;
    }
}
exports.getProductIdFromVariantId = getProductIdFromVariantId;
async function getProductIdsFromVariantIds(variantIds, limit = 475, wait = 20000) {
    try {
        const reqInput = [...variantIds];
        const results = {};
        let failuresCounts = 0;
        while (reqInput.length) {
            const tempInput = [];
            while (tempInput.length < limit + 1 && reqInput.length) {
                const val = reqInput.pop();
                if (val)
                    tempInput.push(val);
            }
            const createQuery = (id) => `
          _${id}: productVariant(id: "gid://shopify/ProductVariant/${id}") {
            product {
              id
            }
          }
        `;
            const query = `
          {
            ${tempInput.map((id) => createQuery(id)).join(" ")}
          }
        `;
            const res2 = await api_1.graphqlClient.request(query).catch((e) => {
                failuresCounts++;
                console.error(e);
            });
            if (res2) {
                for (const [vId, value] of Object.entries(res2)) {
                    if (vId && value) {
                        results[vId.replace("_", "")] = value.product.id;
                    }
                    else {
                        console.log(vId, value);
                    }
                }
            }
            if (wait === 0)
                continue;
            await (0, promises_1.setTimeout)(wait);
        }
        console.log({ failuresCounts });
        return results;
    }
    catch (e) {
        throw e;
    }
}
exports.getProductIdsFromVariantIds = getProductIdsFromVariantIds;
async function createProduct(input) {
    const mutation = (0, api_1.gql) `
    mutation productCreate($input: ProductInput!) {
      productCreate(input: $input) {
        product {
          id
        }
        userErrors {
          field
          message
        }
      }
    }
  `;
    return await api_1.graphqlClient.request(mutation, input);
}
exports.createProduct = createProduct;
async function productExists(sku) {
    const query = (0, api_1.gql) `
  {
    products(first:1,query:"sku:${sku}") {
      nodes {
        id
      }
    }
  }
  `;
    const res = await api_1.graphqlClient.request(query);
    if (res.products.nodes?.length) {
        return { exists: true, productId: res.products.nodes[0].id };
    }
    return false;
}
exports.productExists = productExists;
async function getAllProductImages() {
    const query = `
  query {
    products {
      edges {
        node {
          id
          images {
            edges {
              node {
                id
                altText
                url
              }
            }
          }
        }
      }
    }
  }
  `;
    return await (0, bulk_operations_1.bulkQuery)(query);
}
exports.getAllProductImages = getAllProductImages;
async function getAllProductVariantIds() {
    const query = `
  query {
    productVariants {
      edges {
        node {
          id
          sku
          product {
            id
          }
        }
      }
    }
  }
  `;
    return await (0, bulk_operations_1.bulkQuery)(query);
}
exports.getAllProductVariantIds = getAllProductVariantIds;
